#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>

#define BUFF_SIZE 123456

void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

/* Program demonstrira postavljanje O_NONBLOCK flega.
   Za uputstvo o raznim pokretanjima programa pogledati vezbe. */
int main(){
  char buffer[BUFF_SIZE];
  int flags;
  int left;
  char *pointer;

  /* Citaju se podaci sa standardnog ulaza. */
  if ( (left = read (STDIN_FILENO, buffer, sizeof (buffer))) < 0)
    error_fatal ("read error");

  /* Postavlja se da ispis na standardni izlaz bude neblokirajuci. */
  if ( (flags = fcntl (STDOUT_FILENO, F_GETFL, NULL)) < 0)
    error_fatal ("get flags error");
  flags = flags | O_NONBLOCK;
  if (fcntl (STDOUT_FILENO, F_SETFL, flags) < 0)
    error_fatal ("get flags error");
  
  /* Dok god je ostalo jos neispisanih karaktera na standardni izlaz,
     pokusava se sa njihovim ispisom. */
  pointer = buffer;
  while (left > 0){
    int written;
    errno = 0;
    written = write (STDOUT_FILENO, pointer, left);
    fprintf (stderr, "Written %d bytes, errno = %d\n", written, errno);

    if (written > 0){
      pointer += written;
      left -= written;
    }
  }

  /* Iskljucuje se neblokirajuci rezim za standardni izlaz;
     u ovom primeru iskljucivanje nema znacaja, navedeno je 
     zbog ilustracije postupka. */
  if ( (flags = fcntl (STDOUT_FILENO, F_GETFL, NULL)) < 0)
    error_fatal ("get flags error");
  flags = flags & ~O_NONBLOCK;
  if (fcntl (STDOUT_FILENO, F_SETFL, flags) < 0)
    error_fatal ("get flags error");

  return 0;
}

